@@ -110,6 +110,7 @@ import org.apache.pdfbox.filter.FilterFactory;
 import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.common.PDPageLabels;
+import org.apache.pdfbox.pdmodel.encryption.AccessPermission;
 import org.apache.pdfbox.pdmodel.encryption.InvalidPasswordException;
 import org.apache.pdfbox.pdmodel.font.PDType1Font;
 import org.apache.pdfbox.pdmodel.graphics.color.PDDeviceCMYK;
@@ -124,6 +125,7 @@ import org.apache.pdfbox.printing.PDFPageable;
  * @author Ben Litchfield
  * @author Khyrul Bashar
  */
+@SuppressWarnings({"serial","squid:MaximumInheritanceDepth"})
 public class PDFDebugger extends JFrame
 {
     private static final Set<COSName> SPECIALCOLORSPACES =
@@ -224,8 +226,10 @@ public class PDFDebugger extends JFrame
             }
         });
         
+        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
         jScrollPane1.setBorder(new BevelBorder(BevelBorder.RAISED));
-        jScrollPane1.setPreferredSize(new Dimension(300, 500));
+        jScrollPane1.setPreferredSize(new Dimension(screenSize.width / 8, 500));
+        jSplitPane1.setDividerLocation(screenSize.width / 8);
         tree.addTreeSelectionListener(new TreeSelectionListener()
         {
             @Override
@@ -239,8 +243,8 @@ public class PDFDebugger extends JFrame
 
         jSplitPane1.setRightComponent(jScrollPane2);
         jSplitPane1.setDividerSize(3);
-        
-        jScrollPane2.setPreferredSize(new Dimension(300, 500));
+
+        jScrollPane2.setPreferredSize(new Dimension(screenSize.width / 8 * 7, 500));
         jScrollPane2.setViewportView(jTextPane1);
 
         jSplitPane1.setLeftComponent(jScrollPane1);
@@ -265,8 +269,7 @@ public class PDFDebugger extends JFrame
         menuBar.add(createViewMenu());
         setJMenuBar(menuBar);
 
-        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
-        setBounds((screenSize.width-700)/2, (screenSize.height-600)/2, 700, 600);
+        setBounds(screenSize.width / 4, screenSize.height / 4, screenSize.width / 2, screenSize.height / 2);
 
         // drag and drop to open files
         setTransferHandler(new TransferHandler()
@@ -1122,6 +1125,12 @@ public class PDFDebugger extends JFrame
         {
             return;
         }
+        AccessPermission ap = document.getCurrentAccessPermission();
+        if (!ap.canPrint())
+        {
+            JOptionPane.showMessageDialog(this, "You do not have permission to print");
+            return;
+        }
 
         try
         {
@@ -1180,17 +1189,6 @@ public class PDFDebugger extends JFrame
      */
     public static void main(String[] args) throws Exception
     {
-        try
-        {
-            // force KCMS (faster than LCMS) if available
-            Class.forName("sun.java2d.cmm.kcms.KcmsServiceProvider");
-            System.setProperty("sun.java2d.cmm", "sun.java2d.cmm.kcms.KcmsServiceProvider");
-        }
-        catch (ClassNotFoundException e)
-        {
-            // do nothing. Happens with JDK7 (which has KCMS) and with JDK10 (which hasn't)
-        }
-
         UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
         if (System.getProperty("apple.laf.useScreenMenuBar") == null)
         {
