@@ -20,6 +20,8 @@
 package org.apache.pdfbox.examples.signature.cert;
 
 import java.io.IOException;
+import java.io.InputStream;
+import java.net.URL;
 import java.security.GeneralSecurityException;
 import java.security.InvalidKeyException;
 import java.security.PublicKey;
@@ -27,22 +29,28 @@ import java.security.SignatureException;
 import java.security.cert.CertPathBuilder;
 import java.security.cert.CertPathBuilderException;
 import java.security.cert.CertStore;
+import java.security.cert.Certificate;
 import java.security.cert.CertificateException;
+import java.security.cert.CertificateFactory;
 import java.security.cert.CollectionCertStoreParameters;
 import java.security.cert.PKIXBuilderParameters;
 import java.security.cert.PKIXCertPathBuilderResult;
 import java.security.cert.TrustAnchor;
 import java.security.cert.X509CertSelector;
 import java.security.cert.X509Certificate;
+import java.security.cert.X509Extension;
 import java.util.Calendar;
+import java.util.Collection;
 import java.util.Date;
 import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.Set;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.encryption.SecurityProvider;
 import org.bouncycastle.asn1.ASN1ObjectIdentifier;
+import org.bouncycastle.asn1.ASN1Primitive;
 import org.bouncycastle.asn1.ASN1Sequence;
 import org.bouncycastle.asn1.DEROctetString;
 import org.bouncycastle.asn1.DERTaggedObject;
@@ -105,11 +113,19 @@ public final class CertificateVerifier
                 throw new CertificateVerificationException("The certificate is self-signed.");
             }
 
+            Set<X509Certificate> certSet = CertificateVerifier.downloadExtraCertificates(cert);
+            int downloadSize = certSet.size();
+            certSet.addAll(additionalCerts);
+            if (downloadSize > 0)
+            {
+                LOG.info("CA issuers: " + (certSet.size() - additionalCerts.size()) + " downloaded certificate(s) are new");
+            }
+
             // Prepare a set of trust anchors (set of root CA certificates)
             // and a set of intermediate certificates
             Set<X509Certificate> intermediateCerts = new HashSet<X509Certificate>();
             Set<TrustAnchor> trustAnchors = new HashSet<TrustAnchor>();
-            for (X509Certificate additionalCert : additionalCerts)
+            for (X509Certificate additionalCert : certSet)
             {
                 if (isSelfSigned(additionalCert))
                 {
@@ -132,7 +148,7 @@ public final class CertificateVerifier
 
             LOG.info("Certification chain verified successfully");
 
-            checkRevocations(cert, additionalCerts, signDate);
+            checkRevocations(cert, certSet, signDate);
 
             return verifiedCertChain;
         }
@@ -241,6 +257,84 @@ public final class CertificateVerifier
         }
     }
 
+    /**
+     * Download extra certificates from the URI mentioned in id-ad-caIssuers in the "authority
+     * information access" extension. The method is lenient, i.e. catches all exceptions.
+     *
+     * @param ext an X509 object that can have extensions.
+     *
+     * @return a certificate set, never null.
+     */
+    public static Set<X509Certificate> downloadExtraCertificates(X509Extension ext)
+    {
+        // https://tools.ietf.org/html/rfc2459#section-4.2.2.1
+        // https://tools.ietf.org/html/rfc3280#section-4.2.2.1
+        // https://tools.ietf.org/html/rfc4325
+        Set<X509Certificate> resultSet = new HashSet<X509Certificate>();
+        byte[] authorityExtensionValue = ext.getExtensionValue(Extension.authorityInfoAccess.getId());
+        if (authorityExtensionValue == null)
+        {
+            return resultSet;
+        }
+        ASN1Primitive asn1Prim;
+        try
+        {
+            asn1Prim = JcaX509ExtensionUtils.parseExtensionValue(authorityExtensionValue);
+        }
+        catch (IOException ex)
+        {
+            LOG.warn(ex.getMessage(), ex);
+            return resultSet;
+        }
+        if (!(asn1Prim instanceof ASN1Sequence))
+        {
+            LOG.warn("ASN1Sequence expected, got " + asn1Prim.getClass().getSimpleName());
+            return resultSet;
+        }
+        ASN1Sequence asn1Seq = (ASN1Sequence) asn1Prim;
+        Enumeration<?> objects = asn1Seq.getObjects();
+        while (objects.hasMoreElements())
+        {
+            // AccessDescription
+            ASN1Sequence obj = (ASN1Sequence) objects.nextElement();
+            ASN1ObjectIdentifier oid = (ASN1ObjectIdentifier) obj.getObjectAt(0);
+            if (!oid.equals(X509ObjectIdentifiers.id_ad_caIssuers))
+            {
+                continue;
+            }
+            DERTaggedObject location = (DERTaggedObject) obj.getObjectAt(1);
+            DEROctetString uri = (DEROctetString) location.getObject();
+            InputStream in = null;
+            try
+            {
+                String urlString = new String(uri.getOctets());
+                LOG.info("CA issuers URL: " + urlString);
+                in = new URL(urlString).openStream();
+                CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
+                Collection<? extends Certificate> altCerts = certFactory.generateCertificates(in);
+                for (Certificate altCert : altCerts)
+                {
+                    resultSet.add((X509Certificate) altCert);
+                }
+                LOG.info("CA issuers URL: " + altCerts.size() + " certificate(s) downloaded");
+            }
+            catch (IOException ex)
+            {
+                LOG.warn(ex.getMessage(), ex);
+            }
+            catch (CertificateException ex)
+            {
+                LOG.warn(ex.getMessage(), ex);
+            }
+            finally
+            {
+                IOUtils.closeQuietly(in);
+            }
+        }
+        LOG.info("CA issuers: Downloaded " + resultSet.size() + " certificate(s) total");
+        return resultSet;
+    }
+
     /**
      * Attempts to build a certification chain for given certificate and to
      * verify it. Relies on a set of root CA certificates (trust anchors) and a
@@ -369,7 +463,7 @@ public final class CertificateVerifier
             return;
         }
 
-        LOG.info("Revocation check of OCSP responder certificate");
+        LOG.info("Check of OCSP responder certificate");
         Set<X509Certificate> additionalCerts2 = new HashSet<X509Certificate>(additionalCerts);
         JcaX509CertificateConverter certificateConverter = new JcaX509CertificateConverter();
         for (X509CertificateHolder certHolder : basicResponse.getCerts())
@@ -388,14 +482,7 @@ public final class CertificateVerifier
                 LOG.error(ex, ex);
             }
         }
-        try
-        {
-            checkRevocations(ocspResponderCertificate, additionalCerts2, now);
-        }
-        catch (GeneralSecurityException ex)
-        {
-            throw new CertificateVerificationException(ex.getMessage(), ex);
-        }
-        LOG.info("Revocation check of OCSP responder certificate done");
+        CertificateVerifier.verifyCertificate(ocspResponderCertificate, additionalCerts2, true, now);
+        LOG.info("Check of OCSP responder certificate done");
     }
 }
