@@ -232,7 +232,9 @@ public class COSWriter implements ICOSVisitor, Closeable
     }
 
     /**
-     * COSWriter constructor for incremental updates.
+     * COSWriter constructor for incremental updates. There must be a path of objects that have
+     * {@link COSUpdateInfo#isNeedToBeUpdated()} set, starting from the document catalog. For
+     * signatures this is taken care by PDFBox itself.
      *
      * @param outputStream output stream where the new PDF data will be written. It will be closed
      * when this object is closed.
@@ -426,9 +428,9 @@ public class COSWriter implements ICOSVisitor, Closeable
     protected void doWriteBody(COSDocument doc) throws IOException
     {
         COSDictionary trailer = doc.getTrailer();
-        COSDictionary root = (COSDictionary)trailer.getDictionaryObject( COSName.ROOT );
-        COSDictionary info = (COSDictionary)trailer.getDictionaryObject( COSName.INFO );
-        COSDictionary encrypt = (COSDictionary)trailer.getDictionaryObject( COSName.ENCRYPT );
+        COSDictionary root = trailer.getCOSDictionary(COSName.ROOT);
+        COSDictionary info = trailer.getCOSDictionary(COSName.INFO);
+        COSDictionary encrypt = trailer.getCOSDictionary(COSName.ENCRYPT);
         if( root != null )
         {
             addObjectToWrite( root );
@@ -534,11 +536,11 @@ public class COSWriter implements ICOSVisitor, Closeable
         String headerString;
         if (fdfDocument != null)
         {
-            headerString = "%FDF-"+ Float.toString(fdfDocument.getDocument().getVersion());
+            headerString = "%FDF-"+ Float.toString(doc.getVersion());
         }
         else
         {
-            headerString = "%PDF-"+ Float.toString(pdDocument.getDocument().getVersion());
+            headerString = "%PDF-"+ Float.toString(doc.getVersion());
         }
         getStandardOutput().write( headerString.getBytes(Charsets.ISO_8859_1) );
         
@@ -577,7 +579,13 @@ public class COSWriter implements ICOSVisitor, Closeable
         }
         // Remove a checksum if present
         trailer.removeItem( COSName.DOC_CHECKSUM );
-        
+
+        COSArray idArray = trailer.getCOSArray(COSName.ID);
+        if (idArray != null)
+        {
+            idArray.setDirect(true);
+        }
+
         trailer.accept(this);
     }
 
@@ -883,15 +891,13 @@ public class COSWriter implements ICOSVisitor, Closeable
         {
             actual = ((COSObject)obj).getObject();
         }
-        COSObjectKey key = null;
-        if( actual != null )
+        // PDFBOX-4540: because objectKeys is accessible from outside, it is possible
+        // that a COSObject obj is already in the objectKeys map.
+        COSObjectKey key = objectKeys.get(obj);
+        if( key == null && actual != null )
         {
             key = objectKeys.get(actual);
         }
-        if( key == null )
-        {
-            key = objectKeys.get(obj);
-        }
         if (key == null)
         {
             setNumber(getNumber()+1);
@@ -1355,7 +1361,7 @@ public class COSWriter implements ICOSVisitor, Closeable
             // we don't have path or size, so do the best we can
             md5.update( Long.toString(idTime).getBytes(Charsets.ISO_8859_1) );
 
-            COSDictionary info = (COSDictionary)trailer.getDictionaryObject( COSName.INFO );
+            COSDictionary info = trailer.getCOSDictionary(COSName.INFO);
             if( info != null )
             {
                 for (COSBase cosBase : info.getValues())
@@ -1370,7 +1376,7 @@ public class COSWriter implements ICOSVisitor, Closeable
             idArray = new COSArray();
             idArray.add( firstID );
             idArray.add( secondID );
-            trailer.setItem( COSName.ID, idArray );
+            trailer.setItem(COSName.ID, idArray);
         }
         cosDoc.accept(this);
     }
