@@ -224,7 +224,7 @@ final class FileSystemFontProvider extends FontProvider
 
             // load cached FontInfo objects
             List<FSFontInfo> cachedInfos = loadDiskCache(files);
-            if (cachedInfos != null && cachedInfos.size() > 0)
+            if (cachedInfos != null && !cachedInfos.isEmpty())
             {
                 fontInfoList.addAll(cachedInfos);
             }
@@ -274,10 +274,10 @@ final class FileSystemFontProvider extends FontProvider
     private File getDiskCacheFile()
     {
         String path = System.getProperty("pdfbox.fontcache");
-        if (path == null)
+        if (path == null || !new File(path).isDirectory() || !new File(path).canWrite())
         {
             path = System.getProperty("user.home");
-            if (path == null)
+            if (path == null || !new File(path).isDirectory() || !new File(path).canWrite())
             {
                 path = System.getProperty("java.io.tmpdir");
             }
@@ -475,7 +475,7 @@ final class FileSystemFontProvider extends FontProvider
             }
         }
         
-        if (pending.size() > 0)
+        if (!pending.isEmpty())
         {
             // re-build the entire cache if we encounter un-cached fonts (could be optimised)
             LOG.warn("New fonts found, font cache will be re-built");
