@@ -31,15 +31,18 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.util.Arrays;
 import java.util.List;
+import java.util.StringTokenizer;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.cos.COSFloat;
 import org.apache.pdfbox.cos.COSName;
+import org.apache.pdfbox.cos.COSObject;
 import org.apache.pdfbox.cos.COSStream;
 import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.PDDocument;
+import org.apache.pdfbox.pdmodel.PDResources;
 import org.apache.pdfbox.pdmodel.common.COSArrayList;
 import org.apache.pdfbox.pdmodel.common.PDRange;
 import org.apache.pdfbox.pdmodel.common.PDStream;
@@ -67,6 +70,13 @@ public final class PDICCBased extends PDCIEBasedColorSpace
     // reasons with LittleCMS (LCMS), see PDFBOX-4309
     // WARNING: do not activate this in a conforming reader
     private boolean useOnlyAlternateColorSpace = false;
+    private static final boolean IS_KCMS;
+
+    static
+    {
+        String cmmProperty = System.getProperty("sun.java2d.cmm");
+        IS_KCMS = !isMinJdk8() || "sun.java2d.cmm.kcms.KcmsServiceProvider".equals(cmmProperty);
+    }
 
     /**
      * Creates a new ICC color space with an empty stream.
@@ -83,11 +93,60 @@ public final class PDICCBased extends PDCIEBasedColorSpace
     /**
      * Creates a new ICC color space using the PDF array.
      *
-     * @param iccArray the ICC stream object
+     * @param iccArray the ICC stream object.
+     * @throws IOException if there is an error reading the ICC profile or if the parameter is
+     * invalid.
+     *
+     * @deprecated This will be private in 3.0. Please use
+     * {@link PDICCBased#create(org.apache.pdfbox.cos.COSArray, org.apache.pdfbox.pdmodel.PDResources)}
+     * instead, which supports caching.
+     */
+    @Deprecated
+    public PDICCBased(COSArray iccArray) throws IOException
+    {
+        checkArray(iccArray);
+        useOnlyAlternateColorSpace = System
+                .getProperty("org.apache.pdfbox.rendering.UseAlternateInsteadOfICCColorSpace") != null;
+        array = iccArray;
+        stream = new PDStream((COSStream) iccArray.getObject(1));
+        loadICCProfile();
+    }
+
+    /**
+     * Creates a new ICC color space using the PDF array, optionally using a resource cache.
+     *
+     * @param iccArray the ICC stream object.
+     * @param resources resources to use as cache, or null for no caching.
+     * @return an ICC color space.
      * @throws IOException if there is an error reading the ICC profile or if the parameter is
      * invalid.
      */
-    public PDICCBased(COSArray iccArray) throws IOException
+    public static PDICCBased create(COSArray iccArray, PDResources resources) throws IOException
+    {
+        checkArray(iccArray);
+        COSBase base = iccArray.get(1);
+        COSObject indirect = null;
+        if (base instanceof COSObject)
+        {
+            indirect = (COSObject) base;
+        }
+        if (indirect != null && resources != null && resources.getResourceCache() != null)
+        {
+            PDColorSpace space = resources.getResourceCache().getColorSpace(indirect);
+            if (space != null && space instanceof PDICCBased)
+            {
+                return (PDICCBased) space;
+            }
+        }
+        PDICCBased space = new PDICCBased(iccArray);
+        if (indirect != null && resources != null && resources.getResourceCache() != null)
+        {
+            resources.getResourceCache().put(indirect, space);
+        }
+        return space;
+    }
+
+    private static void checkArray(COSArray iccArray) throws IOException
     {
         if (iccArray.size() < 2)
         {
@@ -97,11 +156,6 @@ public final class PDICCBased extends PDCIEBasedColorSpace
         {
             throw new IOException("ICCBased colorspace array must have a stream as second element");
         }
-        useOnlyAlternateColorSpace = System
-            .getProperty("org.apache.pdfbox.rendering.UseAlternateInsteadOfICCColorSpace") != null;
-        array = iccArray;
-        stream = new PDStream((COSStream) iccArray.getObject(1));
-        loadICCProfile();
     }
 
     @Override
@@ -168,15 +222,20 @@ public final class PDICCBased extends PDCIEBasedColorSpace
                 }
                 initialColor = new PDColor(initial, this);
 
-                // do things that trigger a ProfileDataException
-                // or CMMException due to invalid profiles, see PDFBOX-1295 and PDFBOX-1740
-                // or ArrayIndexOutOfBoundsException, see PDFBOX-3610
-                awtColorSpace.toRGB(new float[awtColorSpace.getNumComponents()]);
-                // this one triggers an exception for PDFBOX-3549 with KCMS
-                new Color(awtColorSpace, new float[getNumberOfComponents()], 1f);
-                // PDFBOX-4015: this one triggers "CMMException: LCMS error 13" with LCMS
-                new ComponentColorModel(awtColorSpace, false, false, 
-                                        Transparency.OPAQUE, DataBuffer.TYPE_BYTE);
+                if (IS_KCMS)
+                {
+                    // do things that trigger a ProfileDataException
+                    // or CMMException due to invalid profiles, see PDFBOX-1295 and PDFBOX-1740 (ü-file)
+                    // or ArrayIndexOutOfBoundsException, see PDFBOX-3610
+                    // also triggers a ProfileDataException for PDFBOX-3549 with KCMS
+                    new Color(awtColorSpace, new float[getNumberOfComponents()], 1f);
+                }
+                else
+                {
+                    // PDFBOX-4015: this one triggers "CMMException: LCMS error 13" with LCMS
+                    new ComponentColorModel(awtColorSpace, false, false,
+                            Transparency.OPAQUE, DataBuffer.TYPE_BYTE);
+                }
             }
         }
         catch (ProfileDataException e)
@@ -510,4 +569,26 @@ public final class PDICCBased extends PDCIEBasedColorSpace
     {
         return getName() + "{numberOfComponents: " + getNumberOfComponents() + "}";
     }
+
+    private static boolean isMinJdk8()
+    {
+        // strategy from lucene-solr/lucene/core/src/java/org/apache/lucene/util/Constants.java
+        String version = System.getProperty("java.specification.version");
+        final StringTokenizer st = new StringTokenizer(version, ".");
+        try
+        {
+            int major = Integer.parseInt(st.nextToken());
+            int minor = 0;
+            if (st.hasMoreTokens())
+            {
+                minor = Integer.parseInt(st.nextToken());
+            }
+            return major > 1 || (major == 1 && minor >= 8);
+        }
+        catch (NumberFormatException nfe)
+        {
+            // maybe some new numbering scheme in the 22nd century
+            return true;
+        }
+    }
 }
