@@ -589,12 +589,13 @@ public abstract class PDAnnotation implements COSObjectable
 
     /**
      * This will get the key of this annotation in the structural parent tree.
-     * 
-     * @return the integer key of the annotation's entry in the structural parent tree
+     *
+     * @return the integer key of the annotation's entry in the structural parent tree or -1 if
+     * there isn't any.
      */
     public int getStructParent()
     {
-        return getCOSObject().getInt(COSName.STRUCT_PARENT, 0);
+        return getCOSObject().getInt(COSName.STRUCT_PARENT);
     }
 
     /**
@@ -773,4 +774,14 @@ public abstract class PDAnnotation implements COSObjectable
         return null;
     }
 
+    /**
+     * Create the appearance entry for this annotation. Not having it may prevent display in some
+     * viewers. This method is for overriding in subclasses, the default implementation does
+     * nothing.
+     * 
+     */
+    public void constructAppearances()
+    {
+    }
+    
 }
