@@ -17,10 +17,16 @@
 package org.apache.pdfbox.rendering;
 
 import java.awt.Color;
+import java.awt.DisplayMode;
 import java.awt.Graphics2D;
+import java.awt.GraphicsConfiguration;
+import java.awt.GraphicsDevice;
+import java.awt.RenderingHints;
 import java.awt.image.BufferedImage;
 import java.io.IOException;
 import java.util.StringTokenizer;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.pdfbox.cos.COSName;
@@ -29,6 +35,8 @@ import org.apache.pdfbox.pdmodel.PDPage;
 import org.apache.pdfbox.pdmodel.PDResources;
 import org.apache.pdfbox.pdmodel.common.PDRectangle;
 import org.apache.pdfbox.pdmodel.graphics.blend.BlendMode;
+import org.apache.pdfbox.pdmodel.graphics.optionalcontent.PDOptionalContentGroup;
+import org.apache.pdfbox.pdmodel.graphics.optionalcontent.PDOptionalContentProperties;
 import org.apache.pdfbox.pdmodel.graphics.state.PDExtendedGraphicsState;
 import org.apache.pdfbox.pdmodel.interactive.annotation.AnnotationFilter;
 import org.apache.pdfbox.pdmodel.interactive.annotation.PDAnnotation;
@@ -60,8 +68,12 @@ public class PDFRenderer
 
     private boolean subsamplingAllowed = false;
 
+    private RenderDestination defaultDestination;
+
+    private RenderingHints renderingHints = null;
+
     private BufferedImage pageImage;
-    
+
     private static boolean kcmsLogged = false;
 
     /**
@@ -78,7 +90,7 @@ public class PDFRenderer
             kcmsLogged = true;
         }
     }
-    
+
     /**
      * Return the AnnotationFilter.
      * 
@@ -129,6 +141,44 @@ public class PDFRenderer
         this.subsamplingAllowed = subsamplingAllowed;
     }
 
+    /**
+     * @return the defaultDestination
+     */
+    public RenderDestination getDefaultDestination()
+    {
+        return defaultDestination;
+    }
+
+    /**
+     * @param defaultDestination the defaultDestination to set
+     */
+    public void setDefaultDestination(RenderDestination defaultDestination)
+    {
+        this.defaultDestination = defaultDestination;
+    }
+
+    /**
+     * Get the rendering hints.
+     *
+     * @return the rendering hints or null if none are set.
+     */
+    public RenderingHints getRenderingHints()
+    {
+        return renderingHints;
+    }
+
+    /**
+     * Set the rendering hints. Use this to influence rendering quality and speed. If you don't set
+     * them yourself or pass null, PDFBox will decide <b><u>at runtime</u></b> depending on the
+     * destination.
+     *
+     * @param renderingHints
+     */
+    public void setRenderingHints(RenderingHints renderingHints)
+    {
+        this.renderingHints = renderingHints;
+    }
+
     /**
      * Returns the given page as an RGB image at 72 DPI
      * @param pageIndex the zero-based index of the page to be converted.
@@ -189,6 +239,22 @@ public class PDFRenderer
      */
     public BufferedImage renderImage(int pageIndex, float scale, ImageType imageType)
             throws IOException
+    {
+        return renderImage(pageIndex, scale, imageType, 
+                           defaultDestination == null ? RenderDestination.EXPORT : defaultDestination);
+    }
+
+    /**
+     * Returns the given page as an RGB or ARGB image at the given scale.
+     * @param pageIndex the zero-based index of the page to be converted
+     * @param scale the scaling factor, where 1 = 72 DPI
+     * @param imageType the type of image to return
+     * @param destination controlling visibility of optional content groups
+     * @return the rendered page image
+     * @throws IOException if the PDF cannot be read
+     */
+    public BufferedImage renderImage(int pageIndex, float scale, ImageType imageType, RenderDestination destination)
+            throws IOException
     {
         PDPage page = document.getPage(pageIndex);
 
@@ -200,6 +266,13 @@ public class PDFRenderer
         int widthPx = (int) Math.max(Math.floor(widthPt * scale), 1);
         int heightPx = (int) Math.max(Math.floor(heightPt * scale), 1);
 
+        // PDFBOX-4518 the maximum size (w*h) of a buffered image is limited to Integer.MAX_VALUE
+        if ((long) widthPx * (long) heightPx > Integer.MAX_VALUE)
+        {
+            throw new IOException("Maximum size of image exceeded (w * h * scale) = "//
+                    + widthPt + " * " + heightPt + " * " + scale + " > " + Integer.MAX_VALUE);
+        }
+
         int rotationAngle = page.getRotation();
 
         int bimType = imageType.toBufferedImageType();
@@ -240,7 +313,10 @@ public class PDFRenderer
         transform(g, page, scale, scale);
 
         // the end-user may provide a custom PageDrawer
-        PageDrawerParameters parameters = new PageDrawerParameters(this, page, subsamplingAllowed);
+        RenderingHints actualRenderingHints =
+                renderingHints == null ? createDefaultRenderingHints(g) : renderingHints;
+        PageDrawerParameters parameters = new PageDrawerParameters(this, page, subsamplingAllowed,
+                                                                   destination, actualRenderingHints);
         PageDrawer drawer = createPageDrawer(parameters);
         drawer.drawPage(g, page.getCropBox());       
         
@@ -297,6 +373,23 @@ public class PDFRenderer
      */
     public void renderPageToGraphics(int pageIndex, Graphics2D graphics, float scaleX, float scaleY)
             throws IOException
+    {
+        renderPageToGraphics(pageIndex, graphics, scaleX, scaleY, 
+                             defaultDestination == null ? RenderDestination.VIEW : defaultDestination);
+    }
+
+    /**
+     * Renders a given page to an AWT Graphics2D instance.
+     * 
+     * @param pageIndex the zero-based index of the page to be converted
+     * @param graphics the Graphics2D on which to draw the page
+     * @param scaleX the scale to draw the page at for the x-axis
+     * @param scaleY the scale to draw the page at for the y-axis
+     * @param destination controlling visibility of optional content groups
+     * @throws IOException if the PDF cannot be read
+     */
+    public void renderPageToGraphics(int pageIndex, Graphics2D graphics, float scaleX, float scaleY, RenderDestination destination)
+            throws IOException
     {
         PDPage page = document.getPage(pageIndex);
         // TODO need width/wight calculations? should these be in PageDrawer?
@@ -307,11 +400,25 @@ public class PDFRenderer
         graphics.clearRect(0, 0, (int) cropBox.getWidth(), (int) cropBox.getHeight());
 
         // the end-user may provide a custom PageDrawer
-        PageDrawerParameters parameters = new PageDrawerParameters(this, page, subsamplingAllowed);
+        RenderingHints actualRenderingHints =
+                renderingHints == null ? createDefaultRenderingHints(graphics) : renderingHints;
+        PageDrawerParameters parameters = new PageDrawerParameters(this, page, subsamplingAllowed,
+                                                                   destination, actualRenderingHints);
         PageDrawer drawer = createPageDrawer(parameters);
         drawer.drawPage(graphics, cropBox);
     }
 
+    /**
+     * Indicates whether an optional content group is enabled.
+     * @param group the group
+     * @return true if the group is enabled
+     */
+    public boolean isGroupEnabled(PDOptionalContentGroup group)
+    {
+        PDOptionalContentProperties ocProperties = document.getDocumentCatalog().getOCProperties();
+        return ocProperties == null || ocProperties.isGroupEnabled(group);
+    }
+
     // scale rotate translate
     private void transform(Graphics2D graphics, PDPage page, float scaleX, float scaleY)
     {
@@ -341,10 +448,43 @@ public class PDFRenderer
                     break;
             }
             graphics.translate(translateX, translateY);
-            graphics.rotate((float) Math.toRadians(rotationAngle));
+            graphics.rotate(Math.toRadians(rotationAngle));
         }
     }
 
+    private boolean isBitonal(Graphics2D graphics)
+    {
+        GraphicsConfiguration deviceConfiguration = graphics.getDeviceConfiguration();
+        if (deviceConfiguration == null)
+        {
+            return false;
+        }
+        GraphicsDevice device = deviceConfiguration.getDevice();
+        if (device == null)
+        {
+            return false;
+        }
+        DisplayMode displayMode = device.getDisplayMode();
+        if (displayMode == null)
+        {
+            return false;
+        }
+        return displayMode.getBitDepth() == 1;
+    }
+
+    private RenderingHints createDefaultRenderingHints(Graphics2D graphics)
+    {
+        RenderingHints r = new RenderingHints(null);
+        r.put(RenderingHints.KEY_INTERPOLATION, isBitonal(graphics) ?
+                RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR :
+                RenderingHints.VALUE_INTERPOLATION_BICUBIC);
+        r.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
+        r.put(RenderingHints.KEY_ANTIALIASING, isBitonal(graphics) ?
+                                        RenderingHints.VALUE_ANTIALIAS_OFF :
+                                        RenderingHints.VALUE_ANTIALIAS_ON);
+        return r;
+    }
+
     /**
      * Returns a new PageDrawer instance, using the given parameters. May be overridden.
      */
@@ -401,18 +541,49 @@ public class PDFRenderer
                 // Make sure that class exists
                 Class.forName("sun.java2d.cmm.kcms.KcmsServiceProvider");
 
-                LOG.info("To get higher rendering speed on java 8 or 9,");
+                String version = System.getProperty("java.version");
+                if (version == null ||
+                    isGoodVersion(version, "1.8.0_(\\d+)", 191) ||
+                    isGoodVersion(version, "9.0.(\\d+)", 4))
+                {
+                    return;
+                }
+                LOG.info("Your current java version is: " + version);
+                LOG.info("To get higher rendering speed on old java 1.8 or 9 versions,");
+                LOG.info("  update to the latest 1.8 or 9 version (>= 1.8.0_191 or >= 9.0.4),");
+                LOG.info("  or");
                 LOG.info("  use the option -Dsun.java2d.cmm=sun.java2d.cmm.kcms.KcmsServiceProvider");
                 LOG.info("  or call System.setProperty(\"sun.java2d.cmm\", \"sun.java2d.cmm.kcms.KcmsServiceProvider\")");
             }
             catch (ClassNotFoundException e)
             {
-                // jdk 10 and higher
-                LOG.debug("KCMS doesn't exist anymore. SO SAD!");
+                // KCMS not available
             }
         }
     }
 
+    private static boolean isGoodVersion(String version, String regex, int min)
+    {
+        Matcher matcher = Pattern.compile(regex).matcher(version);
+        if (matcher.matches() && matcher.groupCount() >= 1)
+        {
+            try
+            {
+                int v = Integer.parseInt(matcher.group(1));
+                if (v >= min)
+                {
+                    // LCMS no longer bad
+                    return true;
+                }
+            }
+            catch (NumberFormatException ex)
+            {
+                return true;
+            }
+        }
+        return false;
+    }
+
     private static boolean isMinJdk8()
     {
         // strategy from lucene-solr/lucene/core/src/java/org/apache/lucene/util/Constants.java
